/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: WidgetModuleMgr.h,v 2.2 1999/10/16 01:44:29 jeff Exp $

*******************************************************************************/


#ifndef WidgetModuleMgr_Included
#define WidgetModuleMgr_Included

#include <qdict.h>
#include <qlist.h>

class DlgWidget;
class WidgetModule;
class QApplication;
class QStrList;

#ifdef _WIN32
#include <windows.h>
typedef HMODULE WidgetModuleHandle;
#else
typedef void* WidgetModuleHandle;
#endif

class WidgetModuleMgr
{
public:

    /**
     *	Load the widget modules
     */
    static void loadModules();

    
    /**
     *	Get the list of widget modules
     */
    static const QList< WidgetModule >& getModules();
    

    /**
     *	Create a new dialog widget based on the ids specified.  
     *
     *	@param	theModuleNum		the number of the module for the new widget
     *	@param	theModuleWidgetNum	the number of the widget in the module
     *	@param	theParent		the parent widget for the new widget
     */
    static DlgWidget* newWidget(
	int		theModuleNum,
	int		theModuleWidgetNum,
	QWidget*	theParent
	);


    /**
     *	Create a new dialog widget based on the names specified.
     *
     *	@param	theModuleName		the name of the module for the new widget
     *	@param	theModuleWidgetName	the name of the widget in the module
     *	@param	theParent		the parent widget for the new widget
     */
    static DlgWidget* newWidget(
	const QString&	theModuleName,
	const QString&	theModuleWidgetName,
	QWidget*	theParent
	);


    /**
     *	Get the QApplication from the module specified in the configuration.
     *
     *	@param	theArgc	the argc
     *	@param	theArgv	the argv
     *
     *	@return a QApplication if a module has been configured to return it,
     *		0 otherwise
     */
    static QApplication* getQApplication(
	int&	theArgc,
	char**	theArgv
	);


    /**
     *	Destructor
     */
    ~WidgetModuleMgr();


private:

    /**
     *	Constructor
     */
    WidgetModuleMgr();
    

    /**
     *	Load the widget modules
     */
    void _loadModules();


    /**
     *	Create a new dialog widget based on the ids specified.  
     *
     *	@param	theModuleNum		the number of the module for the new widget
     *	@param	theModuleWidgetNum	the number of the widget in the module
     *	@param	theParent		the parent widget for the new widget
     */
    DlgWidget* _newWidget(
	int		theModuleNum,
	int		theModuleWidgetNum,
	QWidget*	theParent
	);


    /**
     *	Create a new dialog widget based on the names specified.
     *
     *	@param	theModuleName		the name of the module for the new widget
     *	@param	theModuleWidgetName	the name of the widget in the module
     *	@param	theParent		the parent widget for the new widget
     */
    DlgWidget* _newWidget(
	const QString&	theModuleName,
	const QString&	theModuleWidgetName,
	QWidget*	theParent
	);


    /**
     *	Get the QApplication from the module specified in the configuration.
     *
     *	@param	theArgc	the argc
     *	@param	theArgv	the argv
     *
     *	@return a QApplication if a module has been configured to return it,
     *		0 otherwise
     */
    QApplication* _getQApplication(
	int&	theArgc,
	char**	theArgv
	);


    /**
     *	Place the list of modules to load int theNames
     *
     *	@param	theNames	the list of module names
     */
    void getModuleNames(
	QStrList&	theNames
	);


    /**
     *	Load the module with the specified name
     *
     *	@param	theModuleName	the name of the module to load
     *	@param	theModuleHandle	the handle of the module loaded
     *
     *	@return	TRUE if the module was successfully loaded
     */
    bool loadModule(
	const QString&		theModuleName,
	WidgetModuleHandle&	theModuleHandle
	);


    /**
     *	Unload the module with the specified handle
     *
     *	@param	theHandle	the handle to the module to unload
     */
    void unloadModule(
	WidgetModuleHandle	theHandle
	);

    /**
     *	Retrieve the WidgetModule from the module
     *
     *	@param	theHandle	the handle of the module to load from
     *
     *	@return	a WidgetModule if successful, 0 if an error occurred
     */
    WidgetModule* getWidgetModule(
	WidgetModuleHandle	theHandle
	);

    
    /**
     *	Load the specified function from the handle specified
     *
     *	@param	theHandle	the handle of the module
     *	@param	theFunctionName	the name of the function to load
     *
     *	@return	a pointer to the function from the module, or 0 if an error
     *		occurred
     */
    void* loadFunction(
	WidgetModuleHandle	theHandle,
	const char*		theFunctionName
	);
    

private:

    QList< WidgetModuleHandle > myModuleHandles;
    // List of handles to the modules

    QList< WidgetModule > myWidgetModules;
    // List of WidgetModules

    QDict< WidgetModule > myWidgetModuleDict;
    // Dictionary mapping the module's name to the module

    static WidgetModuleMgr itsInstance;
    // Singleton instance
};


inline void WidgetModuleMgr::loadModules()
{
    itsInstance._loadModules();
}


inline DlgWidget* WidgetModuleMgr::newWidget(
    int		theModuleNum,
    int		theModuleWidgetNum,
    QWidget*	theParent
    )
{
    return itsInstance._newWidget( theModuleNum, 
				   theModuleWidgetNum, 
				   theParent );
}


inline DlgWidget* WidgetModuleMgr::newWidget(
    const QString&	theModuleName,
    const QString&	theModuleWidgetName,
    QWidget*		theParent
    )
{
    return itsInstance._newWidget( theModuleName,
				   theModuleWidgetName,
				   theParent );
}


inline QApplication* WidgetModuleMgr::getQApplication(
    int&	theArgc,
    char**	theArgv
    )
{
    return itsInstance._getQApplication( theArgc, theArgv );
}


inline const QList< WidgetModule >& WidgetModuleMgr::getModules()
{
    return itsInstance.myWidgetModules;
}


#endif // WidgetModuleMgr_Included
