/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: BoxLayout.h,v 2.4 2000/04/29 20:15:46 klaus Exp $

*******************************************************************************/


#ifndef BoxLayout_included
#define BoxLayout_included

#include "layout/LayoutItem.h"

class DlgMenuBar;
class QtArchGenerator;

class BoxLayout : public LayoutItem
{
public: // Methods

    BoxLayout(
	LayoutItem*	theParent
	);
    // Constructor

    BoxLayout(
	QListView*	theParent
	);
    // Constructor

    virtual ~BoxLayout();
    // Destructor

    virtual LayoutItemPropPage* GetPropertyPage( QWidget*	parent,
						 bool		topLevel,
						 const WidgetLayout&	theWidgetLayout );
    // Get the property page for the item.  All the information on the page 
    // should be updated.  The caller is responsible for freeing the
    // page when finished.

    //
    // Layout creation
    //

    virtual QLayout* RecreateLayout( QWidget*	theBaseWindow );
    // Create a new layout for theBaseWindow and add all of its children to it.

    virtual void AddToLayout( QBoxLayout*	theBoxLayout );
    // Add the item and all its children to a box layout.

    virtual void AddToLayout( QGridLayout*	theGridLayout,
			      int		theRowNum,
			      int		theColNum );
    // Add the item and all its children to a grid layout.


    //
    // Loading and Saving
    //

    virtual void SaveContents( QtArchGenerator & ) const;
    // Save the contents of this layout item to theStream with theId as
    // its position identifier.

    virtual void Restore( const XMLValue &, LayoutHelper * );
    // Restore the item from the given XMLValue
    
    virtual bool RestoreKeyValue( const QString&	theKey,
				  const QString&	theValue,
				  const DlgWidget*	theDlgWidget );
    // Restore a key/value pair for the item.  theDlgWidget is the dialog
    // widget associated with theValue, if any.


    //
    // Code generation
    //

    virtual void GenerateSource( QTextStream&	theStream,
				 const QString&	theLayoutId = "1",
				 const QString&	theLayoutParentWidget = QString() ) const;
    // Generate the source for the entire layout tree.

    virtual void GenerateSourceInBox( QTextStream&	theStream,
				      const QString&	theLayoutVarName,
				      const QString&	theLayoutId,
				      const QString&	theLayoutParentWidget ) const;
    // Generate the source code in a box layout.


    virtual void GenerateSourceInGrid( QTextStream&	theStream,
				       const QString&	theLayoutVarName,
				       const QString&	theLayoutId,
				       unsigned int	theRowNum,
				       unsigned int	theColNum,
				       const QString&	theLayoutParentWidget ) const;
    // Generate the source code in a grid layout.


    //
    // Get/Set Properties
    //

    // Direction
    QBoxLayout::Direction GetDirection( void ) const;
    void SetDirection( QBoxLayout::Direction	direction );
    
    // Border
    int GetBorder( void ) const;
    void SetBorder( int	border );
    
    // Auto Border
    int GetAutoBorder( void ) const;
    void SetAutoBorder( int	autoBorder );
    
    // Name
    const QString& GetName( void ) const;
    void SetName( const QString&	name );
    
    // Strut
    int GetStrut( void ) const;
    void SetStrut( int	strut );
    
    // Stretch
    int GetStretch( void ) const;
    void SetStretch( int	stretch );
    
    // MenuBar widget
    const DlgMenuBar* GetMenuBarWidget( void ) const;
    void SetMenuBarWidget( const DlgMenuBar*	menuWidget );

private: // Methods

    void SetLayoutContents( QBoxLayout*	theLayout );
    // Set the properties and the children of theLayout.

    void GenerateCtor( QTextStream&	theStream,
		       const QString&	theVarName,
		       bool		isTopLevel,
		       const QString&	theLayoutParentWidget ) const;
    // Generate the constructor for the box layout.

    void GenerateContents( QTextStream&		theStream,
			   const QString&	theVarName,
			   const QString&	theLayoutId,
			   const QString&	theLayoutParentWidget ) const;
    // Generate the contents of the box layout

private: // Class Methods

    static QString GetTreeLabel();
    // Get the label for the item in the tree
    
private: // Attributes

    //
    // Properties
    //

    QBoxLayout::Direction	myDirection;
    int				myBorder;
    int				myAutoBorder;
    QString			myName;
    int				myStrut;
    int				myStretch;
    const DlgMenuBar*		myMenuBarWidget;

    static QString	myTreeLabel;
    // Label for the item in the tree

};


inline QBoxLayout::Direction BoxLayout::GetDirection( void ) const
{
    return myDirection;
}


inline void BoxLayout::SetDirection( QBoxLayout::Direction	direction )
{
    myDirection = direction;
}

    
inline int BoxLayout::GetBorder( void ) const
{
    return myBorder;
}


inline void BoxLayout::SetBorder( int	border )
{
    myBorder = border;
}

    
inline int BoxLayout::GetAutoBorder( void ) const
{
    return myAutoBorder;
}


inline void BoxLayout::SetAutoBorder( int	autoBorder )
{
    myAutoBorder = autoBorder;
}

    
inline const QString& BoxLayout::GetName( void ) const
{
    return myName;
}


inline int BoxLayout::GetStrut( void ) const
{
    return myStrut;
}


inline void BoxLayout::SetStrut( int	strut )
{
    myStrut = strut;
}

    
inline int BoxLayout::GetStretch( void ) const
{
    return myStretch;
}


inline void BoxLayout::SetStretch( int	stretch )
{
    myStretch = stretch;
}

    
inline const DlgMenuBar* BoxLayout::GetMenuBarWidget( void ) const
{
    return myMenuBarWidget;
}


inline void BoxLayout::SetMenuBarWidget( const DlgMenuBar*	menuWidget )
{
    myMenuBarWidget = menuWidget;
}


#endif // BoxLayout_included
